from enum import StrEnum


class QueryID(StrEnum):
    user = "2e2e3b656d2ba48e0b2cd5eeedf88ef70e4aabb4ac4d9d9e9b8feff343a37d98"
    deals = "c3b623b5fe0758cf91b2335ebf36ff65f8650a6672a792a3ca7a36d270d396fb"
    deal = "5652037a966d8da6d41180b0be8226051fe0ed1357d460c6ae348c3138a0fba3"
    games = "5de9b3240c148579c82e2310a30b4aad5462884fd1abf93dd3c43d1f5ef14d85"
    game = "4775f8630a3e234c50537e68649043ac32a40b0370b0f1fb2dc314500ef6202d"
    game_category = "d81943c23bc558591f70286ad69bb6bf7f6229d04aae39fb0a9701d78a9fd749"
    game_category_agreements = "3ea4b047196ed9f84aa5eb652299c4bd73f2e99e9fdf4587877658d9ea6330f6"
    game_category_obtaining_types = (
        "15b0991414821528251930b4c8161c299eb39882fd635dd5adb1a81fb0570aea"
    )
    game_category_instructions = "5991cead6a8ca46195bc4f7ae3164e7606105dbb82834c910658edeb0a1d1918"
    game_category_data_fields = "6fdadfb9b05880ce2d307a1412bc4f2e383683061c281e2b65a93f7266ea4a49"
    chats = "999f86b7c94a4cb525ed5549d8f24d0d24036214f02a213e8fd7cefc742bbd58"
    chat = "bb024dc0652fc7c1302a64a117d56d99fb0d726eb4b896ca803dca55f611d933"
    chat_messages = "e8162a8500865f4bb18dbaacb1c4703823f74c1925a91a5103f41c2021f0557a"
    items = "206ae9d63e58bc41df9023aae39b9136f358282a808c32ee95f5b8b6669a8c8b"
    item = "5b2be2b532cea7023f4f584512c4677469858e2210349f7eec78e3b96d563716"
    item_priority_statuses = "b922220c6f979537e1b99de6af8f5c13727daeff66727f679f07f986ce1c025a"
    transaction_providers = "31960e5dd929834c1f85bc685db80657ff576373076f016b2578c0a34e6e9f42"
    transactions = "3b9925106c3fe9308ac632254fd70da347b5701f243ab8690477d5a7ca37c2c8"
    sbp_bank_members = "ef7902598e855fa15fb5e3112156ac226180f0b009a36606fc80a18f00b80c63"
    verified_cards = "eb338d8432981307a2b3d322b3310b2447cab3a6acf21aba4b8773b97e72d1aa"


class GameType(StrEnum):
    GAME = "GAME"
    APPLICATION = "APPLICATION"


class UserType(StrEnum):
    USER = "USER"
    MODERATOR = "MODERATOR"
    BOT = "BOT"


class GameCategoryDataFieldTypes(StrEnum):
    ITEM_DATA = "ITEM_DATA"
    OBTAINING_DATA = "OBTAINING_DATA"


class EventTypes(StrEnum):
    CHAT_INITIALIZED = "CHAT_INITIALIZED"
    NEW_MESSAGE = "NEW_MESSAGE"
    NEW_DEAL = "NEW_DEAL"
    NEW_REVIEW = "NEW_REVIEW"
    DEAL_CONFIRMED = "DEAL_CONFIRMED"
    DEAL_CONFIRMED_AUTOMATICALLY = "DEAL_CONFIRMED_AUTOMATICALLY"
    DEAL_ROLLED_BACK = "DEAL_ROLLED_BACK"
    DEAL_HAS_PROBLEM = "DEAL_HAS_PROBLEM"
    DEAL_PROBLEM_RESOLVED = "DEAL_PROBLEM_RESOLVED"
    DEAL_STATUS_CHANGED = "DEAL_STATUS_CHANGED"
    ITEM_PAID = "ITEM_PAID"
    ITEM_SENT = "ITEM_SENT"


class ItemLogEvents(StrEnum):
    PAID = "PAID"
    SENT = "SENT"
    DEAL_CONFIRMED = "DEAL_CONFIRMED"
    DEAL_ROLLED_BACK = "DEAL_ROLLED_BACK"
    PROBLEM_REPORTED = "PROBLEM_REPORTED"
    PROBLEM_RESOLVED = "PROBLEM_RESOLVED"


class TransactionOperations(StrEnum):
    DEPOSIT = "DEPOSIT"
    BUY = "BUY"
    SELL = "SELL"
    ITEM_DEFAULT_PRIORITY = "ITEM_DEFAULT_PRIORITY"
    ITEM_PREMIUM_PRIORITY = "ITEM_PREMIUM_PRIORITY"
    WITHDRAW = "WITHDRAW"
    MANUAL_BALANCE_INCREASE = "MANUAL_BALANCE_INCREASE"
    MANUAL_BALANCE_DECREASE = "MANUAL_BALANCE_DECREASE"
    REFERRAL_BONUS = "REFERRAL_BONUS"
    STEAM_DEPOSIT = "STEAM_DEPOSIT"


class TransactionDirections(StrEnum):
    IN = "IN"
    OUT = "OUT"


class TransactionStatuses(StrEnum):
    PENDING = "PENDING"
    PROCESSING = "PROCESSING"
    CONFIRMED = "CONFIRMED"
    ROLLED_BACK = "ROLLED_BACK"
    FAILED = "FAILED"


class TransactionPaymentMethodIds(StrEnum):
    MIR = "MIR"
    VISA_MASTERCARD = "VISA_MASTERCARD"
    ERIP = "ERIP"


class TransactionProviderDirections(StrEnum):
    IN = "IN"
    OUT = "OUT"


class TransactionProviderIds(StrEnum):
    LOCAL = "LOCAL"
    SBP = "SBP"
    BANK_CARD_RU = "BANK_CARD_RU"
    BANK_CARD_BY = "BANK_CARD_BY"
    BANK_CARD = "BANK_CARD"
    YMONEY = "YMONEY"
    USDT = "USDT"
    PENDING_INCOME = "PENDING_INCOME"


class BankCardTypes(StrEnum):
    MIR = "MIR"
    VISA = "VISA"
    MASTERCARD = "MASTERCARD"


class ItemDealStatuses(StrEnum):
    PAID = "PAID"
    PENDING = "PENDING"
    SENT = "SENT"
    CONFIRMED = "CONFIRMED"
    ROLLED_BACK = "ROLLED_BACK"


class ItemDealDirections(StrEnum):
    IN = "IN"
    OUT = "OUT"


class ChatTypes(StrEnum):
    PM = "PM"
    NOTIFICATIONS = "NOTIFICATIONS"
    SUPPORT = "SUPPORT"


class ChatStatuses(StrEnum):
    NEW = "NEW"
    FINISHED = "FINISHED"


class ChatMessageButtonTypes(StrEnum):
    # TODO: Add another types of buttons
    REDIRECT = "REDIRECT"
    LOTTERY = "LOTTERY"
    ASK_FOR_EXTERNAL_REVIEW = "ASK_FOR_EXTERNAL_REVIEW"


class ItemStatuses(StrEnum):
    PENDING_APPROVAL = "PENDING_APPROVAL"
    PENDING_MODERATION = "PENDING_MODERATION"
    APPROVED = "APPROVED"
    DECLINED = "DECLINED"
    BLOCKED = "BLOCKED"
    EXPIRED = "EXPIRED"
    SOLD = "SOLD"
    DRAFT = "DRAFT"


class ReviewStatuses(StrEnum):
    APPROVED = "APPROVED"
    DELETED = "DELETED"


class SortDirections(StrEnum):
    DESC = "DESC"
    ASC = "ASC"


class ItemsSortOptions(StrEnum):
    PRICE_ASC = "PRICE_ASC"
    PRICE_DESC = "PRICE_DESC"
    RATING_ASC = "RATING_ASC"
    RATING_DESC = "RATING_DESC"
    DEFAULT = "DEFAULT"


class PriorityTypes(StrEnum):
    DEFAULT = "DEFAULT"
    PREMIUM = "PREMIUM"


class GameCategoryAgreementIconTypes(StrEnum):
    # TODO: Add another types of icons
    RESTRICTION = "RESTRICTION"
    CONFIRMATION = "CONFIRMATION"


class GameCategoryOptionTypes(StrEnum):
    # TODO: Add another types of options
    SELECTOR = "SELECTOR"
    SWITCH = "SWITCH"


class GameCategoryDataFieldInputTypes(StrEnum):
    # Maybe, another obtaining types can be exists
    INPUT = "INPUT"
    TEXTAREA = "TEXTAREA"


class GameCategoryAutoConfirmPeriods(StrEnum):
    # TODO: Add all confirm periods
    SEVEN_DAYS = "SEVEN_DAYS"


class GameCategoryInstructionTypes(StrEnum):
    FOR_SELLER = "FOR_SELLER"
    FOR_BUYER = "FOR_BUYER"



from datetime import datetime
from typing import Any

from pydantic import AliasChoices, Field

from . import ApiModel, UserType


class Account(ApiModel):
    id: str = Field(..., alias="id")
    username: str = Field(..., alias="username")
    email: str = Field(..., alias="email")
    role: UserType = Field(..., alias="role")
    has_frozen_balance: bool = Field(..., alias="hasFrozenBalance")
    support_chat_id: str = Field(..., alias="supportChatId")
    system_chat_id: str = Field(..., alias="systemChatId")
    unread_chats_counter: int = Field(..., alias="unreadChatsCounter")
    is_blocked: bool = Field(..., alias="isBlocked")
    is_blocked_for: Any = Field(None, alias="isBlockedFor")
    created_at: datetime = Field(..., alias="createdAt")
    last_item_created_at: datetime = Field(None, alias="lastItemCreatedAt")
    has_confirmed_phone_number: bool = Field(..., alias="hasConfirmedPhoneNumber")
    can_publish_items: bool = Field(..., alias="canPublishItems")
    is_funds_protection_active: bool = Field(..., alias="isFundsProtectionActive")
    chosen_verified_card: Any = Field(None, alias="chosenVerifiedCard")


class UserProfile(ApiModel):
    id: str = Field(..., alias="id")
    username: str = Field(..., alias="username")
    role: UserType = Field(UserType.USER, alias="role")
    avatar_url: str | None = Field(None, alias="avatarURL")
    is_online: bool | None = Field(None, alias="isOnline")
    is_blocked: bool = Field(False, alias="isBlocked")
    rating: float | None = Field(None, alias="rating")
    reviews_count: int | None = Field(
        None,
        alias="reviewsCount",
        validation_alias=AliasChoices("reviewsCount", "testimonialCounter"),
    )
    created_at: datetime | None = Field(None, alias="createdAt")


class AccountBalance(ApiModel):
    id: str = Field(..., alias="id")
    value: float = Field(..., alias="value")  # Total balance
    available: float = Field(..., alias="available")  # Unlocked balance for purchasing items
    frozen: float = Field(..., alias="frozen")  # idk
    pending_income: float = Field(
        ..., alias="pendingIncome"
    )  # Unlockable balance (after up to 48 hours). Can be wasted for premium
    withdrawable: float = Field(
        ..., alias="withdrawable"
    )  # Balance that can be withdrawn right now.


class AccountItemsStats(ApiModel):
    total: int = Field(..., alias="total")
    finished: int = Field(..., alias="finished")


class AccountIncomingDealsStats(ApiModel):
    total: int = Field(..., alias="total")
    finished: int = Field(..., alias="finished")


class AccountOutgoingDealsStats(ApiModel):
    total: int = Field(..., alias="total")
    finished: int = Field(..., alias="finished")


class AccountDealsStats(ApiModel):
    incoming: AccountIncomingDealsStats = Field(..., alias="incoming")
    outgoing: AccountOutgoingDealsStats = Field(..., alias="outgoing")


class AccountStats(ApiModel):
    items: AccountItemsStats = Field(..., alias="items")
    deals: AccountDealsStats = Field(..., alias="deals")


class AccountProfile(UserProfile):
    email: str = Field(..., alias="email")
    balance: AccountBalance = Field(..., alias="balance")
    stats: AccountStats = Field(..., alias="stats")
    is_blocked_for: Any = Field(None, alias="isBlockedFor")
    is_verified: bool | None = Field(None, alias="isVerified")
    has_frozen_balance: bool = Field(..., alias="hasFrozenBalance")
    has_enabled_notifications: bool = Field(..., alias="hasEnabledNotifications")
    support_chat_id: str = Field(..., alias="supportChatId")
    system_chat_id: str = Field(..., alias="systemChatId")



from __future__ import annotations

from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from ..schemas.deals import ItemDeal as SchemaDeal
from ..schemas.enums import ItemDealStatuses

if TYPE_CHECKING:  # pragma: no cover
    from ..playerok import Playerok
    from .chat import Chat
    from .user import User


@dataclass(slots=True)
class Deal:
    id: str
    status: ItemDealStatuses | None = None

    _client: "Playerok" = field(repr=False, default=None)
    _user_id: str | None = field(repr=False, default=None)
    _chat_id: str | None = field(repr=False, default=None)

    @classmethod
    def _create(cls, *, client: "Playerok", id: str) -> "Deal":
        return cls(id=id, _client=client)

    def _merge_schema(self, deal: SchemaDeal) -> None:
        self.status = deal.status

        if deal.user is not None:
            self._user_id = deal.user.id
            self._client._push_user_profile(deal.user)

        if deal.chat is not None:
            self._chat_id = deal.chat.id

    @property
    def user(self) -> "User | None":
        if not self._user_id:
            return None
        return self._client._get_user_identity(self._user_id)

    @property
    def chat(self) -> "Chat | None":
        if not self._chat_id:
            return None
        return self._client._get_chat_identity(self._chat_id)

    async def refresh(self) -> "Deal":
        await self._client._push_deal(self.id)
        return self

    async def complete(self) -> "Deal":
        # minimal "complete": CONFIRMED (IN) or SENT (OUT) depends on direction; we keep it simple.
        updated = await self._client.raw.deals.update_deal(self.id, ItemDealStatuses.CONFIRMED)
        if updated is not None:
            self._client._push_deal_schema(updated)
        return self



from __future__ import annotations

from ..core.utils import _dig, _raise_on_gql_errors
from ..graphql import GraphQLQuery as GQL
from ..schemas import (
    ItemDeal,
    ItemDealDirections,
    ItemDealList,
    ItemDealStatuses,
    Transaction,
    TransactionPaymentMethodIds,
    TransactionProviderIds,
)
from ..transport import PlayerokTransport


class RawDealsService:
    def __init__(self, transport: PlayerokTransport):
        self._transport = transport

    async def get_deals(
        self,
        user_id: str,
        count: int = 24,
        statuses: list[ItemDealStatuses] | None = None,
        direction: ItemDealDirections | None = None,
        after_cursor: str | None = None,
    ) -> ItemDealList | None:
        response = await self._transport.request(
            "post",
            "graphql",
            GQL.get_deals(
                count=count,
                user_id=user_id,
                statuses=statuses,
                direction=direction,
                after_cursor=after_cursor,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "deals"))
        if data is None:
            return None
        return ItemDealList(**data)

    async def get_deal(self, deal_id: str) -> ItemDeal | None:
        response = await self._transport.request("post", "graphql", GQL.get_deal(deal_id=deal_id))
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "deal"))
        if data is None:
            return None
        return ItemDeal(**data)

    async def update_deal(self, deal_id: str, new_status: ItemDealStatuses) -> ItemDeal | None:
        response = await self._transport.request(
            "post", "graphql", GQL.update_deal(deal_id=deal_id, new_status=new_status)
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "updateDeal"))
        if data is None:
            return None
        return ItemDeal(**data)

    async def create_deal(
        self,
        item_id: str,
        transaction_provider_id: TransactionProviderIds,
        obtaining_fields: dict[str, str] | None = None,
        comment_from_buyer: str | None = None,
        payment_method_id: TransactionPaymentMethodIds | None = None,
    ) -> Transaction | None:
        payload_obtaining_fields = (
            [{"fieldId": k, "value": v} for k, v in obtaining_fields.items()]
            if obtaining_fields
            else None
        )

        response = await self._transport.request(
            "post",
            "graphql",
            GQL.create_deal(
                item_id=item_id,
                transaction_provider_id=transaction_provider_id.name,
                obtaining_fields=payload_obtaining_fields,
                comment_from_buyer=comment_from_buyer,
                payment_method_id=payment_method_id.name if payment_method_id else None,
            ),
        )
        raw = response.json()
        _raise_on_gql_errors(raw)

        data = _dig(raw, ("data", "createDeal"))
        if data is None:
            return None
        return Transaction(**data)



from datetime import datetime

from pydantic import Field, model_validator

from . import (
    ApiModel,
    GameCategoryAgreementIconTypes,
    GameCategoryAutoConfirmPeriods,
    GameCategoryDataFieldInputTypes,
    GameCategoryDataFieldTypes,
    GameCategoryOptionTypes,
    GameType,
    PageInfo,
)
from .basic import File


class GameCategoryDataField(ApiModel):
    id: str = Field(..., alias="id")
    label: str = Field(..., alias="label")

    type: GameCategoryDataFieldTypes = Field(..., alias="type")
    input_type: GameCategoryDataFieldInputTypes = Field(..., alias="inputType")

    copyable: bool = Field(..., alias="copyable")
    hidden: bool = Field(..., alias="hidden")
    required: bool = Field(..., alias="required")

    value: str | None = Field(None, alias="value")


class GameCategoryDataFieldList(ApiModel):
    data_fields: list[GameCategoryDataField] = Field(..., alias="dataFields")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "dataFields" not in data and "data_fields" not in data:
            edges = data.get("edges") or []
            data["dataFields"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]

        if "data_fields" in data and "dataFields" not in data:
            data["dataFields"] = data["data_fields"]

        return data


class GameCategoryProps(ApiModel):
    min_reviews: int | None = Field(None, alias="minTestimonials")
    min_reviews_for_seller: int = Field(..., alias="minTestimonialsForSeller")


class GameCategoryOption(ApiModel):
    id: str = Field(..., alias="id")
    group: str = Field(..., alias="group")
    label: str = Field(..., alias="label")

    type: GameCategoryOptionTypes = Field(..., alias="type")
    field: str = Field(..., alias="field")
    value: str = Field(..., alias="value")
    value_range_limit: int | None = Field(None, alias="valueRangeLimit")


class GameCategoryAgreement(ApiModel):
    id: str = Field(..., alias="id")
    description: str = Field(..., alias="description")
    icon_type: GameCategoryAgreementIconTypes = Field(..., alias="iconType")
    sequence: int = Field(..., alias="sequence")


class GameCategoryAgreementList(ApiModel):
    agreements: list[GameCategoryAgreement] = Field(..., alias="agreements")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "agreements" not in data:
            edges = data.get("edges") or []
            data["agreements"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]
        return data


class GameCategoryObtainingType(ApiModel):
    id: str = Field(..., alias="id")
    name: str = Field(..., alias="name")
    description: str = Field(..., alias="description")

    game_category_id: str = Field(..., alias="gameCategoryId")
    no_comment_from_buyer: bool = Field(..., alias="noCommentFromBuyer")
    instruction_for_buyer: str | None = Field(None, alias="instructionForBuyer")
    instruction_for_seller: str | None = Field(None, alias="instructionForSeller")

    sequence: int = Field(..., alias="sequence")
    fee_multiplier: float | None = Field(None, alias="feeMultiplier")

    agreements: list[GameCategoryAgreement] = Field(..., alias="agreements")
    props: GameCategoryProps | None = Field(None, alias="props")


class GameCategoryObtainingTypeList(ApiModel):
    obtaining_types: list[GameCategoryObtainingType] = Field(..., alias="obtainingTypes")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "obtainingTypes" not in data and "obtaining_types" not in data:
            edges = data.get("edges") or []
            data["obtainingTypes"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]

        if "obtaining_types" in data and "obtainingTypes" not in data:
            data["obtainingTypes"] = data["obtaining_types"]

        return data


class GameCategoryInstruction(ApiModel):
    id: str = Field(..., alias="id")
    text: str = Field(..., alias="text")


class GameCategoryInstructionList(ApiModel):
    instructions: list[GameCategoryInstruction] = Field(..., alias="instructions")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "instructions" not in data:
            edges = data.get("edges") or []
            data["instructions"] = [
                (edge or {}).get("node") for edge in edges if (edge or {}).get("node")
            ]
        return data


class GameCategory(ApiModel):
    id: str = Field(..., alias="id")
    slug: str = Field(..., alias="slug")
    name: str = Field(..., alias="name")

    category_id: str | None = Field(None, alias="categoryId")
    game_id: str | None = Field(None, alias="gameId")

    obtaining: str | None = Field(None, alias="obtaining")

    options: list[GameCategoryOption] | None = Field(None, alias="options")
    props: GameCategoryProps | None = Field(None, alias="props")

    no_comment_from_buyer: bool | None = Field(None, alias="noCommentFromBuyer")
    instruction_for_buyer: str | None = Field(None, alias="instructionForBuyer")
    instruction_for_seller: str | None = Field(None, alias="instructionForSeller")

    use_custom_obtaining: bool | None = Field(None, alias="useCustomObtaining")
    auto_confirm_period: GameCategoryAutoConfirmPeriods | None = Field(
        None, alias="autoConfirmPeriod"
    )
    auto_moderation_mode: bool | None = Field(None, alias="autoModerationMode")

    agreements: list[GameCategoryAgreement] | None = Field(None, alias="agreements")
    fee_multiplier: float | None = Field(None, alias="feeMultiplier")


class Game(ApiModel):
    id: str = Field(..., alias="id")
    slug: str = Field(..., alias="slug")
    name: str = Field(..., alias="name")

    type: GameType = Field(..., alias="type")

    logo: File = Field(..., alias="logo")
    banner: File | None = Field(None, alias="banner")

    categories: list[GameCategory] = Field(..., alias="categories")
    created_at: datetime = Field(..., alias="createdAt")


class GameProfile(ApiModel):
    id: str = Field(..., alias="id")
    slug: str = Field(..., alias="slug")
    name: str = Field(..., alias="name")
    type: GameType = Field(..., alias="type")
    logo: File = Field(..., alias="logo")


class GameList(ApiModel):
    games: list[Game] = Field(..., alias="games")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "games" not in data:
            edges = data.get("edges") or []
            data["games"] = [(edge or {}).get("node") for edge in edges if (edge or {}).get("node")]
        return data



from datetime import datetime
from typing import Any

from pydantic import Field, model_validator

from . import ApiModel, ItemStatuses, PageInfo, PriorityTypes, UserType
from .account import UserProfile
from .basic import File


class ItemPriorityStatusPriceRange(ApiModel):
    min: int | None = Field(None, alias="min")
    max: int | None = Field(None, alias="max")


class ItemPriorityStatus(ApiModel):
    id: str = Field(..., alias="id")
    price: int = Field(..., alias="price")
    name: str = Field(..., alias="name")
    type: PriorityTypes = Field(..., alias="type")
    period: int = Field(..., alias="period")
    price_range: ItemPriorityStatusPriceRange = Field(..., alias="priceRange")


class ItemLog(ApiModel):
    id: str = Field(..., alias="id")
    event: str = Field(..., alias="event")  # Using str since ItemLogEvents might be missing some
    created_at: datetime = Field(..., alias="createdAt")
    user: UserProfile | None = Field(None, alias="user")


class Item(ApiModel):
    id: str = Field(..., alias="id")
    slug: str | None = Field(None, alias="slug")
    name: str | None = Field(None, alias="name")
    description: str | None = Field(None, alias="description")
    price: int | None = Field(None, alias="price")
    raw_price: int | None = Field(None, alias="rawPrice")

    priority: PriorityTypes | None = Field(None, alias="priority")
    status: ItemStatuses | None = Field(None, alias="status")
    seller_type: UserType | None = Field(None, alias="sellerType")

    attachment: File | None = Field(None, alias="attachment")
    attachments: list[File] | None = Field(None, alias="attachments")
    user: UserProfile | None = Field(None, alias="user")

    attributes: dict[str, Any] | None = Field(None, alias="attributes")
    comment: str | None = Field(None, alias="comment")

    approval_date: datetime | None = Field(None, alias="approvalDate")
    priority_position: int | None = Field(None, alias="priorityPosition")
    views_counter: int | None = Field(None, alias="viewsCounter")
    fee_multiplier: float | None = Field(None, alias="feeMultiplier")
    created_at: datetime | None = Field(None, alias="createdAt")
    updated_at: datetime | None = Field(None, alias="updatedAt")
    deleted_at: datetime | None = Field(None, alias="deletedAt")


class MyItem(Item):
    prev_price: int | None = Field(None, alias="prevPrice")
    prev_fee_multiplier: float | None = Field(None, alias="prevFeeMultiplier")
    seller_notified_about_fee_change: bool | None = Field(
        None, alias="sellerNotifiedAboutFeeChange"
    )
    priority_price: int | None = Field(None, alias="priorityPrice")
    sequence: int | None = Field(None, alias="sequence")
    status_expiration_date: datetime | None = Field(None, alias="statusExpirationDate")
    status_description: str | None = Field(None, alias="statusDescription")
    status_payment: Any | None = Field(None, alias="statusPayment")  # Transaction
    is_editable: bool | None = Field(None, alias="isEditable")
    buyer: UserProfile | None = Field(None, alias="buyer")


class ItemList(ApiModel):
    items: list[Item] = Field(..., alias="items")
    page_info: PageInfo = Field(..., alias="pageInfo")
    total_count: int = Field(..., alias="totalCount")

    @model_validator(mode="before")
    @classmethod
    def _inflate_from_edges(cls, data):
        if not isinstance(data, dict):
            return data

        if "items" not in data:
            edges = data.get("edges") or []
            data["items"] = [(edge or {}).get("node") for edge in edges if (edge or {}).get("node")]

        return data
